<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 21/04/20
 * Time: 12.02
 */

namespace FiloBlu\Rma\Handlers;

use Exception;
use FiloBlu\Rma\Api\RmaExtensionAttributesRepositoryInterface;
use FiloBlu\Rma\Helper\ChangeSizeAttributeHelper;
use FiloBlu\Rma\Helper\ChangeSizeOrder;
use FiloBlu\Rma\Helper\RmaHelper;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Rma\Model\Item as ItemModel;
use Magento\Rma\Model\Rma;
use Magento\Rma\Model\Rma\Source\Status;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;
use Throwable;

/**
 *
 */
class CreateChangeSizeOrderHandler
{
    /**
     * @var array
     */
    protected $changSizeItems = [];
    /**
     * @var ChangeSizeAttributeHelper
     */
    private $changeSizeAttributeHelper;
    /**
     * @var RmaExtensionAttributesRepositoryInterface
     */
    private $rmaExtensionAttributesRepository;
    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var ChangeSizeOrder
     */
    private $changeSizeOrder;

    /**
     * @var RequestInterface
     */
    private $request;

    /**
     * @var ProductRepositoryInterface
     */
    private $productRepository;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var ItemModel
     */
    private $itemModel;
    /**
     * @var RmaHelper
     */
    private $rmaHelper;

    /**
     * CreateChangeSizeOrderHandler constructor.
     * @param ChangeSizeAttributeHelper $changeSizeAttributeHelper
     * @param RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository
     * @param LoggerInterface $logger
     * @param ChangeSizeOrder $changeSizeOrder
     * @param RequestInterface $request
     * @param ProductRepositoryInterface $productRepository
     * @param StoreManagerInterface $storeManager
     * @param ItemModel $itemModel
     * @param RmaHelper $rmaHelper
     */
    public function __construct(
        ChangeSizeAttributeHelper                 $changeSizeAttributeHelper,
        RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository,
        LoggerInterface                           $logger,
        ChangeSizeOrder                           $changeSizeOrder,
        RequestInterface                          $request,
        ProductRepositoryInterface                $productRepository,
        StoreManagerInterface                     $storeManager,
        ItemModel                                 $itemModel,
        RmaHelper                                 $rmaHelper
    )
    {
        $this->changeSizeAttributeHelper = $changeSizeAttributeHelper;
        $this->rmaExtensionAttributesRepository = $rmaExtensionAttributesRepository;
        $this->logger = $logger;
        $this->changeSizeOrder = $changeSizeOrder;
        $this->request = $request;
        $this->productRepository = $productRepository;
        $this->storeManager = $storeManager;
        $this->itemModel = $itemModel;
        $this->rmaHelper = $rmaHelper;
    }


    /**
     * TODO: Refactor
     * @param Rma $subject
     * @param $result
     * @return mixed
     */
    public function checkIfCreateChangeSizeOrder(Rma $subject, $result)
    {
        try {

            if ($result && isset($result['items'])) {

                $rmaItems = $result['items'];
                $parentProduct = null;
                $websiteId = $this->storeManager->getStore($subject->getStoreId())->getWebsiteId();

                foreach ($rmaItems as $item) {

                    if ($item->getStatus() == Status::STATE_PENDING
                        && $item->getResolution() == $this->changeSizeAttributeHelper->getChangeSizeAttributeOptionId()
                        && $this->changeSizeAttributeHelper->isChangeSizeEnabled($websiteId)
                    ) {


                        if (!$this->rmaExtensionAttributesRepository->getChangeSizeOrderId($item->getId())) {

                            $changeSizeItem = [];

                            // check if customer
                            if ($item->getExtensionAttributes() !== null && $item->getExtensionAttributes()->getChangeSize() != null) {
                                $item->setChangeSize($item->getExtensionAttributes()->getChangeSize());
                            }
                            if ($item->getChangeSize() != '' || (isset($item->getData()[ChangeSizeAttributeHelper::CHANGE_SIZE_RMA_ATTRIBUTE_CODE])
                                    && $item->getData()[ChangeSizeAttributeHelper::CHANGE_SIZE_RMA_ATTRIBUTE_CODE] != '')) {

                                $changeProductId = $item->getChangeSize();
                                /** @var ItemModel $item */
                                $parentProduct = $item->getProductSku() ?? $this->itemModel->setEntityId($item->getEntityId())->getProductSku();
                                $changeSizeItem = [
                                    'product_id' => $changeProductId,
                                    'qty' => $item['qty_requested'],
                                    'parent_product_sku' => $parentProduct,
                                    'rma_entity_id' => $item->getEntityId(),
                                    'original_order_item_id' => $item->getOrderItemId()
                                ];
                            }
                            $customEmail = $subject->getCustomerCustomEmail();
                            if (isset($customEmail) && $customEmail != '') {
                                $subject->getOrder()->setCustomerEmail($customEmail);
                            }
                            $isLastItem = true;
                            if (next($rmaItems) == true) {
                                $isLastItem = false;
                            }

                            $changeSizeItem['is_last_item'] = $isLastItem;

                            if ($this->rmaHelper->getChangeSizeOrderCreationStrategy() == 'each')  {
                                // create an order for each item requested
                                $this->createChangeSizeOrder($subject->getOrder(), [$changeSizeItem], $subject->getEntityId(), $parentProduct, $isLastItem);
                            }

                            $this->changSizeItems[] = $changeSizeItem;
                        }
                    }
                }

                if ($this->rmaHelper->getChangeSizeOrderCreationStrategy() == 'total') {
                    //create on order with all items requested
                    $this->createChangeSizeOrder($subject->getOrder(), $this->changSizeItems, $subject->getEntityId(), $parentProduct, true);
                }
            }
        } catch (Exception $exception) {
            $this->logger->critical($exception->getMessage(), ['exception' => $exception]);
            return $result;
        } catch (Throwable $throwable) {
            $this->logger->critical($throwable->getMessage(), ['exception' => $throwable]);
            return $result;
        }
        return $result;
    }


    /**
     * @param $order
     * @param array $items
     * @param $rmaId
     * @param null $parentProduct
     * @param bool $isLastItem
     * @throws \Exception
     */
    protected function createChangeSizeOrder($order, array $items, $rmaId, $parentProduct = null, $isLastItem = true )
    {
        $orderData = [
            'order' => $order,
            'rma_id' => $rmaId,
            'parent_product_sku' => $parentProduct,
            'request' => $this->request->getParams(),
            'items' => $items,
            'is_last_item' => $isLastItem
        ];

        $this->changeSizeOrder->createMageOrder($orderData);
    }

}
